﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace YieldReturnAndUsing
{
    internal interface IDisposableNumbersCollection : IDisposable
    {
        ReadOnlyCollection<INumber> TheList { get; }
    }

    internal sealed class DisposableNumbersCollection : IDisposableNumbersCollection
    {
        private ReadOnlyCollection<INumber> m_TheList;

        private Boolean m_Disposed;

        public DisposableNumbersCollection()
        {
            Tracer.Trace(TraceState.Entering, "DisposableNumbersCollection.ctor");

            m_Disposed = false;

            IList<INumber> theList = new List<INumber>() { new Number(1), new Number(2), new Number(3)
                , new Number(4), new Number(5), new Number(6) };

            m_TheList = new ReadOnlyCollection<INumber>(theList);

            Tracer.Trace(TraceState.Leaving, "DisposableNumbersCollection.ctor");
        }

        #region IDisposableNumbersCollection Members

        ReadOnlyCollection<INumber> IDisposableNumbersCollection.TheList
        {
            get
            {
                Tracer.Trace(TraceState.Entering, "DisposableNumbersCollection.TheList");

                CheckDisposed();

                String typeName = GetTypeName(m_TheList);

                Tracer.Output("Getting the list:\r\n{0}", typeName);

                Tracer.Trace(TraceState.Leaving, "DisposableNumbersCollection.TheList");

                return (m_TheList);
            }
        }

        #endregion

        #region IDisposable Members

        void IDisposable.Dispose()
        {
            Tracer.Trace(TraceState.Entering, "DisposableNumbersCollection.Dispose");

            CheckDisposed();

            Tracer.Output("Now the list will be made invalid. This must happen after the iteration has been completed.");

            m_TheList = null;

            m_Disposed = true;

            Tracer.Trace(TraceState.Leaving, "DisposableNumbersCollection.Dispose");
        }

        #endregion

        private static String GetTypeName(Object obj)
        {
            Tracer.Trace(TraceState.Entering, "DisposableNumbersCollection.GetTypeName");

            String typeName = (obj != null) ? obj.GetType().ToString() : "<null>";

            Tracer.Trace(TraceState.Leaving, "DisposableNumbersCollection.GetTypeName");

            return (typeName);
        }

        private void CheckDisposed()
        {
            Tracer.Trace(TraceState.Entering, "DisposableNumbersCollection.CheckDisposed");

            if(m_Disposed)
            {
                Tracer.Output("Warning: DisposableNumbersCollection is already disposed");
            }

            Tracer.Trace(TraceState.Leaving, "DisposableNumbersCollection.CheckDisposed");
        }
    }
}