﻿using System;
using System.Diagnostics;

namespace YieldReturnAndUsing
{
    internal enum TraceState
    {
        Entering,

        YieldReturning,

        Leaving,
    }

    [DebuggerStepThrough]
    internal static class Tracer
    {
        private static UInt16 s_Indentation;

        static Tracer()
        {
            s_Indentation = 0;
        }

        internal static void Trace(TraceState state
            , String method)
        {
            String stateString;

            if (state == TraceState.Entering)
            {
                stateString = "Entering";
                s_Indentation += 2;
            }
            else if (state == TraceState.YieldReturning)
            {
                stateString = "Yield Returning";
            }
            else
            {
                stateString = "Leaving";
            }

            stateString = stateString.PadRight(16);

            Console.Write("".PadRight(s_Indentation));
            Console.WriteLine("{0}{1}", stateString, method);

            if (state == TraceState.Leaving)
            {
                s_Indentation -= 2;
            }
        }

        internal static void Output(Object text, params Object[] parameters)
        {
            Console.WriteLine();
            Console.WriteLine(text.ToString(), parameters);
            Console.WriteLine();
        }

        internal static void Output(Exception ex)
        {
            Console.WriteLine();
            Console.WriteLine(ex.Message);
            Console.WriteLine(ex.StackTrace);
            Console.WriteLine();
        }
    }
}