﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace YieldReturnAndUsing
{
    internal sealed class WrongEnumerator : IEnumerator<Int32>
    {
        private Int32 m_CurrentIndex;

        private ReadOnlyCollection<INumber> m_TheList;

        public WrongEnumerator(ReadOnlyCollection<INumber> theList)
        {
            Tracer.Trace(TraceState.Entering, "WrongEnumerator.ctor");
            
            m_TheList = theList;
            m_CurrentIndex = -1;

            Tracer.Trace(TraceState.Leaving, "WrongEnumerator.ctor");
        }

        #region IEnumerator<Int32> Members

        Int32 IEnumerator<Int32>.Current
        {
            get
            {
                Tracer.Trace(TraceState.Entering, "WrongEnumerator.IEnumerator<Int32>.Current.get");

                Int32 theNumber = m_TheList[m_CurrentIndex].TheNumber;

                Tracer.Trace(TraceState.Leaving, "WrongEnumerator.IEnumerator<Int32>.Current.get");

                return (theNumber);
            }
        }

        #endregion

        #region IDisposable Members

        void IDisposable.Dispose()
        {
            Tracer.Trace(TraceState.Entering, "WrongEnumerator.Dispose");

            m_TheList = null;

            Tracer.Trace(TraceState.Leaving, "WrongEnumerator.Dispose");
        }

        #endregion

        #region IEnumerator Members

        Object IEnumerator.Current
        {
            get
            {
                Tracer.Trace(TraceState.Entering, "WrongEnumerator.IEnumerator.Current.get");

                IEnumerator<Int32> enumerator = (IEnumerator<Int32>)this;

                Int32 current = enumerator.Current;

                Tracer.Trace(TraceState.Leaving, "WrongEnumerator.IEnumerator.Current.get");

                return (current);
            }
        }

        Boolean IEnumerator.MoveNext()
        {
            Tracer.Trace(TraceState.Entering, "WrongEnumerator.MoveNext");

            try
            {
                m_CurrentIndex++;

                Boolean hasNext = (m_CurrentIndex < m_TheList.Count);

                return (hasNext);
            }
            finally
            {
                Tracer.Trace(TraceState.Leaving, "WrongEnumerator.MoveNext");
            }
        }

        void IEnumerator.Reset()
        {
            Tracer.Trace(TraceState.Entering, "WrongEnumerator.Reset");

            m_CurrentIndex = -1;

            Tracer.Trace(TraceState.Leaving, "WrongEnumerator.Reset");
        }

        #endregion
    }
}